package de.tudarmstadt.maki.simonstrator.api.component.privacy;

import java.util.ArrayList;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

public class PrivacyLevel implements Comparable<PrivacyLevel> {
	public static final PrivacyLevel NO_PRIVACY = new PrivacyLevel(0);
	public static final List<PrivacyLevel> PRIVACY_LEVELS = new ArrayList<>();

	private static int staticID = 0;

	private final int id;
    private double imprecisionAreaRadius;
	private int[] responsibility = new int[0];

	@XMLConfigurableConstructor({ "imprecision" })
	public PrivacyLevel(double pImprecisionAreaRadius) {
		synchronized (PrivacyLevel.class) {
			id = staticID++;
		}
        imprecisionAreaRadius = pImprecisionAreaRadius;
    }

	@XMLConfigurableConstructor({ "imprecision", "responsibility" })
	public PrivacyLevel(double pImprecisionAreaRadius, String responsibility) {
		synchronized (PrivacyLevel.class) {
			id = staticID++;
		}
		imprecisionAreaRadius = pImprecisionAreaRadius;
		String[] responsibilityLevels;
		if (responsibility.contains(";")) {
			responsibilityLevels = responsibility.split(";");
		} else {
			responsibilityLevels = new String[0];
		}
		this.responsibility = new int[responsibilityLevels.length];
		for (int i = 0; i < responsibilityLevels.length; i++) {
			this.responsibility[i] = Integer.valueOf(responsibilityLevels[i]);
		}
	}

    public int getId() {
        return id;
    }

    public double getImprecisionAreaRadius() {
        return imprecisionAreaRadius;
    }

    @Override
    public int hashCode() {
        return id;
    }

	public void setResponsibility(int[] responsibility) {
		this.responsibility = responsibility;
	}

	public boolean isResponsible(int pState) {
		for (int i = 0; i < responsibility.length; i++) {
			if (responsibility[i] == pState) {
				return true;
			}
		}
		return false;
	}

    @Override
    public boolean equals(Object pObj) {
        if (pObj instanceof PrivacyLevel) {
            return ((PrivacyLevel) pObj).id == id;
        }
        return false;
    }

	@Override
	public int compareTo(PrivacyLevel pArg0) {
		return Double.compare(imprecisionAreaRadius, pArg0.getImprecisionAreaRadius());
	}
}
