/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.costs;

import java.util.HashMap;
import java.util.Map;

import de.tudarmstadt.maki.simonstrator.api.component.pubsub.Notification;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.LocationBasedEnvironmentProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RoadProperty;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.EnvironmentInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.RoadInformation;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

public class ConfigurableVehicularPropertyImpactEstimator implements VehicularPropertyImpactEstimator {
	private Map<Class<?>, VehicularPropertyImpact> _properties = new HashMap<>();

	public ConfigurableVehicularPropertyImpactEstimator() {
		PropertyImpactEstimatorFactory.registerPropertyBenefitEstimator(this);
	}

	public void setPropertyImpact(VehicularPropertyImpact pImpact) {
		try {
			_properties.put(Class.forName(pImpact.getProperty()), pImpact);
		} catch (ClassNotFoundException e) {
			throw new AssertionError("Unknown class: ", e);
		}
	}

	@Override
	public double calculateImpactIfKnown(Class<? extends RoadProperty> pProperty) {
		if (_properties.containsKey(pProperty)) {
			return _properties.get(pProperty).getImpactKnown();
		}
		return 0;
	}

	@Override
	public double calculateImpactIfUnknown(Class<? extends RoadProperty> pProperty) {
		if (_properties.containsKey(pProperty)) {
			return _properties.get(pProperty).getImpactUnknown();
		}
		return 0;
	}

	@Override
	public double calculateImpactIfKnown(Notification pNotification) {
		EnvironmentInformation<? extends LocationBasedEnvironmentProperty> environmentInformation = EnvironmentInformation
				.createFromNotification(pNotification);

		if (environmentInformation instanceof RoadInformation) {
			return calculateImpactIfKnown(((RoadInformation) environmentInformation).getValue());
		}

		throw new AssertionError("Cost calculation only valid for road information!");
	}

	@Override
	public double calculateImpactIfUnknown(Notification pNotification) {
		EnvironmentInformation<? extends LocationBasedEnvironmentProperty> environmentInformation = EnvironmentInformation
				.createFromNotification(pNotification);

		if (environmentInformation instanceof RoadInformation) {
			return calculateImpactIfUnknown(((RoadInformation) environmentInformation).getValue());
		}

		throw new AssertionError("Cost calculation only valid for road information!");
	}

	public static class VehicularPropertyImpact {
		private String _property;
		private double _impactKnown;
		private double _impactUnknown;

		@XMLConfigurableConstructor({ "property", "known", "unknown" })
		public VehicularPropertyImpact(String pProperty, double pImpactKnown, double pImpactUnknown) {
			_property = pProperty;
			_impactKnown = pImpactKnown;
			_impactUnknown = pImpactUnknown;
		}

		@XMLConfigurableConstructor({ "property", "known" })
		public VehicularPropertyImpact(String pProperty, double pImpactKnown) {
			_property = pProperty;
			_impactKnown = pImpactKnown;
			_impactUnknown = pImpactKnown;
		}

		public String getProperty() {
			return _property;
		}

		public double getImpactKnown() {
			return _impactKnown;
		}

		public double getImpactUnknown() {
			return _impactUnknown;
		}
	}
}
