/*
 * Copyright (c) 2005-2010 KOM � Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties;

import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.aggregation.AggregationInformation;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

public abstract class AbstractRoadProperty implements RoadProperty, AggregatedProperty {
	private Location _location;
	private RoadNetworkEdge _edge;
	private long _detectionDate;

	private AggregationInformation _aggregationInformation;
	private long _duration;

	public AbstractRoadProperty(Location pLocation, RoadNetworkEdge pEdge) {
		_location = pLocation;
		_edge = pEdge;
		_detectionDate = Time.getCurrentTime();
	}

	@Override
	public long getDetectionDate() {
		return _detectionDate;
	}

	@Override
	public void setDetectionDate(long pDate) {
		_detectionDate = pDate;
	}

	@Override
	public Location getLocation() {
		return _location;
	}

	@Override
	public void setLocation(Location pLocation) {
		_location = pLocation;
	}

	@Override
	public RoadNetworkEdge getEdge() {
		return _edge;
	}

	@Override
	public long getDuration() {
		return _duration;
	}

	@Override
	public void setDuration(long pDuration) {
		_duration = pDuration;
	}

	public void setEdge(RoadNetworkEdge pEdge) {
		_edge = pEdge;
	}

	@Override
	public AggregationInformation getAggregationInformation() {
		return _aggregationInformation;
	}

	@Override
	public void setAggregationInformation(AggregationInformation pAggregationInformation) {
		_aggregationInformation = pAggregationInformation;
	}

	@Override
	public int hashCode() {
		if (_location != null) {
			return _location.hashCode() * 31 + Long.hashCode(getDetectionDate());
		} else {
			return _edge.hashCode() * 31 + Long.hashCode(getDetectionDate());
		}
	}

	@Override
	public boolean isAggregated() {
		return _aggregationInformation != null;
	}

	@Override
	public boolean equalLocation(RoadProperty pRoadProperty) {
		return pRoadProperty.getEdge().equals(_edge) && pRoadProperty.getLocation().equals(_location);
	}

	@Override
	public int compareTo(RoadProperty pO) {
		return Long.compare(getDetectionDate(), pO.getDetectionDate());
	}

	@Override
	public boolean equals(Object pObj) {
		if (pObj instanceof AbstractRoadProperty) {
			AbstractRoadProperty property = (AbstractRoadProperty) pObj;
			if (property.getClass().equals(getClass())) {
				return (_location == property._location || (_location != null && _location.equals(property._location)))
						&& getValue().equals(property.getValue()) && _edge.equals(property._edge)
						&& _detectionDate == property._detectionDate;
			}
		}
		return false;
	}

	@Override
	public boolean isDetourable() {
		return false;
	}

	public abstract AbstractRoadProperty clone();
}
