/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.plugin;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.Randoms;
import de.tudarmstadt.maki.simonstrator.api.component.ComponentNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.EnvironmentProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.JamProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RoadProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSRequest;
import de.tudarmstadt.maki.simonstrator.api.component.sis.exception.InformationNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.sis.type.SiSTypes;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 27.02.2018
 *
 */
public class JamEnvironmentSensorPlugin implements EnvironmentSensorPlugin {
	private Host _host;

	private SiSComponent _sis;

	private double _accuracy;

	private double _range = 200;

	private static Random _random = Randoms.getRandom(JamEnvironmentSensorPlugin.class);

	@XMLConfigurableConstructor({ "accuracy" })
	public JamEnvironmentSensorPlugin(double pAccuracy) {
		setAccuracy(pAccuracy);
	}

	@Override
	public void setHost(Host pHost) {
		_host = pHost;
	}

	public void setRange(double pRange) {
		_range = pRange;
	}

	@Override
	public List<EnvironmentProperty> getEnvironmentProperties() {
		if (_sis == null) {
			try {
				_sis = _host.getComponent(SiSComponent.class);
			} catch (ComponentNotAvailableException e) {
				throw new AssertionError("SiS requried!", e);
			}
		}
		
		try {
			List<RoadNetworkEdge> investigated = new ArrayList<>();
			RoadNetworkEdge edge = _sis.get().localState(SiSTypes.ROAD_EDGE, SiSRequest.NONE);
			investigated.add(edge);

			for (int i = 0; i < 2; i++) {
				Set<RoadNetworkEdge> toBeAdded = new HashSet<>();
				for (RoadNetworkEdge roadNetworkEdge : investigated) {
					toBeAdded.add(roadNetworkEdge);
					toBeAdded.addAll(roadNetworkEdge.getAccessibleEdges());
					toBeAdded.addAll(roadNetworkEdge.getAccessibleEdges());
				}
				investigated = new ArrayList<>(toBeAdded);
			}

			Location location = _sis.get().localState(SiSTypes.PHY_LOCATION, SiSRequest.NONE);

			List<EnvironmentProperty> properties = new ArrayList<>();
			for (RoadNetworkEdge roadNetworkEdge : investigated) {
				boolean jammed = false;
				Location eventLocation = null;
				RoadNetworkEdge eventEdge = null;
				for (RoadProperty roadProperty : roadNetworkEdge.getActiveProperties()) {
					if (roadProperty instanceof JamProperty) {
						if (((JamProperty) roadProperty).isJammed()) {
							jammed = true;
						}
						eventLocation = roadProperty.getLocation();
						eventEdge = roadProperty.getEdge();
					}
				}
				if (_random.nextDouble() >= _accuracy) {
					jammed = !jammed;
				}

				if (eventLocation != null && location.distanceTo(eventLocation) <= _range) {
					EnvironmentProperty property = new JamProperty(eventLocation, eventEdge, jammed);
					properties.add(property);
				}
			}
			return properties;
		} catch (InformationNotAvailableException e) {
			throw new AssertionError(SiSTypes.ROAD_EDGE + " and " + SiSTypes.PHY_LOCATION + " are required!");
		}
	}

	@Override
	public Class<? extends RoadProperty> getSupportedClass() {
		return JamProperty.class;
	}

	@Override
	public JamEnvironmentSensorPlugin clone() throws CloneNotSupportedException {
		JamEnvironmentSensorPlugin jamEnvironmentSensorPlugin = new JamEnvironmentSensorPlugin(_accuracy);
		jamEnvironmentSensorPlugin.setRange(_range);
		return jamEnvironmentSensorPlugin;
	}

	public void setAccuracy(double pAccuracy) {
		_accuracy = pAccuracy;
	}

}
