/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.plugin;

import java.util.List;
import java.util.Random;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.Randoms;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.EnvironmentProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RoadProperty;

public class MaliciousEnvironmentSensorPlugin implements EnvironmentSensorPlugin {
	private static final Random RANDOM = Randoms.getRandom(MaliciousEnvironmentSensorPlugin.class);

	private EnvironmentSensorPlugin _goodPlugin;
	private EnvironmentSensorPlugin _badPlugin;

	private boolean _isMalicious = false;
	private double _probability;

	public void setMaliciousProbability(double pProbability) {
		_probability = pProbability;

		if (RANDOM.nextDouble() < _probability) {
			_isMalicious = true;
		} else {
			_isMalicious = false;
		}
	}

	public void setGoodPlugin(EnvironmentSensorPlugin pGoodPlugin) {
		_goodPlugin = pGoodPlugin;

		if (_badPlugin != null && !_badPlugin.getSupportedClass().equals(_goodPlugin.getSupportedClass())) {
			throw new AssertionError("Both plugins need to provide the same data entry.");
		}
	}

	public void setBadPlugin(EnvironmentSensorPlugin pBadPlugin) {
		_badPlugin = pBadPlugin;

		if (_goodPlugin != null && !_badPlugin.getSupportedClass().equals(_goodPlugin.getSupportedClass())) {
			throw new AssertionError("Both plugins need to provide the same data entry.");
		}
	}

	@Override
	public List<EnvironmentProperty> getEnvironmentProperties() {
		if (_isMalicious) {
			return _badPlugin.getEnvironmentProperties();
		} else {
			return _goodPlugin.getEnvironmentProperties();
		}
	}

	@Override
	public EnvironmentSensorPlugin clone() throws CloneNotSupportedException {
		MaliciousEnvironmentSensorPlugin maliciousEnvironmentSensorPlugin = new MaliciousEnvironmentSensorPlugin();
		maliciousEnvironmentSensorPlugin.setGoodPlugin(_goodPlugin.clone());
		maliciousEnvironmentSensorPlugin.setBadPlugin(_badPlugin.clone());
		maliciousEnvironmentSensorPlugin.setMaliciousProbability(_probability);
		return maliciousEnvironmentSensorPlugin;
	}

	@Override
	public Class<? extends RoadProperty> getSupportedClass() {
		return _goodPlugin.getSupportedClass();
	}

	@Override
	public void setHost(Host pHost) {
		_goodPlugin.setHost(pHost);
		_badPlugin.setHost(pHost);
	}

	public boolean isMalicious() {
		return _isMalicious;
	}

}
