/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sis.type;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 03.01.2018
 *
 */
public class ListBasedPredictionContainer<T extends Object> implements PredictionContainer<T> {
	private List<T> values = new ArrayList<>();

	public ListBasedPredictionContainer(T[] values) {
		for (T t : values) {
			this.values.add(t);
		}
	}

	public ListBasedPredictionContainer(T value) {
		this.values.add(value);
	}

	public ListBasedPredictionContainer(List<T> values) {
		this.values = values;
	}

	public List<T> getValues() {
		return values;
	}

	@Override
	public double getProbabilityForValue(T pValue) {
		if (values.contains(pValue)) {
			return 1 / (double) values.size();
		}
		return 0;
	}

	@Override
	public double isInsidePrediction(T pValue) {
		if (values.contains(pValue)) {
			return 1;
		}
		return 0;
	}

	@Override
	public T getMostProbableValue() {
		return values.get(0);
	}
}
