/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular;

import java.util.ArrayList;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.Environment;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.RoadInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkRoute;

public interface VehicleDecisionComponent extends HostComponent {
	void startDecisionMaking();

	BenefitBasedRoute getOptimalRoute(RoadNetworkRoute pInvestigatedRoute, List<RoadInformation> knownInformation);

	default BenefitBasedRoute getOptimalRoute(RoadNetworkRoute pInvestigatedRoute,
			List<RoadInformation> knownInformation, Environment pEnvironment) {
		return getOptimalRoute(pInvestigatedRoute, knownInformation);
	}

	default BenefitBasedRoute getOptimalRoute(RoadNetworkRoute pInvestigatedRoute, RoadInformation knownInformation) {
		List<RoadInformation> known = new ArrayList<>();
		known.add(knownInformation);
		return getOptimalRoute(pInvestigatedRoute, known);
	}

	class BenefitBasedRoute {
		private final double _benefit;
		private final RoadNetworkRoute _route;

		public BenefitBasedRoute(double pBenefit, RoadNetworkRoute pRoute) {
			_benefit = pBenefit;
			_route = pRoute;
		}

		public double getBenefit() {
			return _benefit;
		}

		public RoadNetworkRoute getRoute() {
			return _route;
		}
	}
}
