/*
 * Copyright (c) 2005-2010 KOM � Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.caching.decision;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.aggregation.AggregationInformation;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RoadProperty;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.PointInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.RoadInformation;

public abstract class AbstractCacheDecisionStrategy implements CacheDecisionStrategy {

	protected void addAggregationInformation(List<? extends PointInformation> pMeasurements,
			PointInformation<?> pAggregate) {
		if (pAggregate instanceof RoadInformation) {
			RoadProperty roadProperty = ((RoadInformation) pAggregate).getValue();

			long minTimestamp = Long.MAX_VALUE;
			long maxTimestamp = 0;

			for (PointInformation<?> t : pMeasurements) {
				long timestamp = t.getDetectionDate();

				if (timestamp < minTimestamp) {
					minTimestamp = timestamp;
				}
				if (timestamp > maxTimestamp) {
					maxTimestamp = timestamp;
				}
			}

			roadProperty.setAggregationInformation(new AggregationInformation(pMeasurements.size(), minTimestamp));
		}
	}

	public abstract CacheDecisionStrategy clone();
}
