package de.tudarmstadt.maki.simonstrator.api.component.vehicular.subscriptions.topic;

import java.util.HashSet;
import java.util.Set;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.component.ComponentNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.pubsub.PubSubComponent;
import de.tudarmstadt.maki.simonstrator.api.component.pubsub.attribute.Topic;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.VehicleInformationComponent;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.PointInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.VehicularPointInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkRoute;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.subscriptions.SubscriptionTopicType;

public class RoadBasedTopic implements ProcessedTopic {

	private Topic _topic;
	private RoadNetworkEdge _edge;

	public RoadBasedTopic(PubSubComponent pPubSub, RoadNetworkEdge pEdge) {
		_topic = pPubSub.createTopic(pEdge.getEdgeID());
		_edge = pEdge;
	}

	@Override
	public Topic getTopic() {
		return _topic;
	}

	@Override
	public SubscriptionTopicType getTopicType() {
		return SubscriptionTopicType.EDGE_BASED;
	}

	@Override
	public boolean matchesInformationRequirements(PointInformation<?> pInformation) {
		if (pInformation instanceof VehicularPointInformation) {
			VehicularPointInformation<?> vehicularInfo = (VehicularPointInformation<?>) pInformation;
			return _edge.equals(vehicularInfo.getEdge());
		}
		return false;
	}

	@Override
	public boolean includes(ProcessedTopic pTopic) {
		return false;
	}

	@Override
	public boolean matchesHost(Host pHost) {
		try {
			VehicleInformationComponent vehicleInfo = pHost.getComponent(VehicleInformationComponent.class);
			RoadNetworkRoute currentRoute = vehicleInfo.getCurrentRoute();
			RoadNetworkEdge currentRoad = currentRoute.getStart();

			Set<RoadNetworkEdge> roads = new HashSet<>();
			roads.add(currentRoad);
			roads.addAll(currentRoad.getAccessibleEdges());

			return roads.contains(_edge);
		} catch (ComponentNotAvailableException e) {
		}
		return false;
	}

}
