/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.web.plotting;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlIDREF;

import de.tudarmstadt.maki.simonstrator.api.web.Metric;
import de.tudarmstadt.maki.simonstrator.api.web.approach.Approach;
import de.tudarmstadt.maki.simonstrator.api.web.parameter.Parameter;

@XmlAccessorType(XmlAccessType.FIELD)
public class Plot {
	public enum DataType {
		PARAMETER, APPROACH, METRIC, NUMBER, STRING
	}

	public enum PlotType {
		TIME, BOX
	}

	@XmlElement(name = "id")
	protected int _plotID = -1;

	@XmlElementWrapper(name = "fixed")
	@XmlElement(name = "value")
	protected Map<Parameter, String> _fixedValues = new HashMap<>();

	@XmlIDREF
	@XmlElementWrapper(name = "metrics")
	@XmlElement(name = "metric")
	protected Metric[] _metrics;

	@XmlIDREF
	@XmlElementWrapper(name = "parameters")
	@XmlElement(name = "parameter")
	protected Parameter[] _parameters;

	@XmlIDREF
	@XmlElementWrapper(name = "approaches")
	@XmlElement(name = "approach")
	protected List<Approach> _approaches = new ArrayList<>();

	@XmlElement(name = "type")
	protected PlotType _type = PlotType.BOX;

	public Plot() {
	}

	public Plot(int pID) {
		_plotID = pID;
	}

	public Metric[] getMetrics() {
		return _metrics;
	}

	public void setMetric(int pIndex, Metric pMetric) {
		_metrics[pIndex] = pMetric;
	}

	public int getNumberOfMetrics() {
		return _metrics.length;
	}

	public Parameter[] getParameters() {
		return _parameters;
	}

	public void setParameter(int pIndex, Parameter pParameter) {
		_parameters[pIndex] = pParameter;
	}

	public int getNumberOfParameters() {
		return _parameters.length;
	}

	public int getPlotID() {
		return _plotID;
	}

	public List<Approach> getApproaches() {
		return _approaches;
	}

	public void addFixedValue(Parameter pParameter, String pValue) {
		_fixedValues.put(pParameter, pValue);
	}

	public void removeFixedValue(Parameter pParameter) {
		_fixedValues.remove(pParameter);
	}

	public Map<Parameter, String> getFixedValues() {
		return Collections.unmodifiableMap(_fixedValues);
	}

	public void addApproach(Approach pApproach) {
		_approaches.add(pApproach);
	}

	public void removeApproach(Approach pApproach) {
		_approaches.remove(pApproach);
	}

	public void addApproaches(List<Approach> pApproaches) {
		_approaches.addAll(pApproaches);
	}

	public PlotType getType() {
		return _type;
	}

	public String getName() {
		switch (_type) {
		case BOX:
			return "Box Plot";
		case TIME:
			return "Time Plot";
		default:
			return _type.toString();
		}
	}

	public Map<String, DataType> getRequiredInput() {
		Map<String, DataType> result = new TreeMap<>();
		for (int i = 0; i < _metrics.length; i++) {
			result.put("Metric " + (i + 1), DataType.METRIC);
		}
		for (int i = 0; i < _parameters.length; i++) {
			result.put("Parameter " + (i + 1), DataType.PARAMETER);
		}
		return result;
	}
}
